use std::fs::File;
use std::io::{self, BufRead};
use std::path::Path;
use std::cmp::Ordering;

#[derive(Debug)]
enum Elem {
    Number(u8),
    List(Vec<Elem>),
}

use crate::Elem::{Number, List};

// I modified my input because the only 2 digits number was 10
// I replaced all 10s with ':' so that I can parse more easely

fn parse_line(line: &str, index: usize) -> (Elem, usize) {
    let mut list: Vec<Elem> = vec![];
    let mut len = 0;
    while index + len < line.len() {
        let c = line.as_bytes()[index + len];
        len += 1;
        match c {
            b'[' => {
                let (sublist, sublist_len) = parse_line(line, index + len);
                len += sublist_len;
                list.push(sublist);
            },
            b']' => return (List(list), len),
            b'0'..=b':' => list.push(Number(c - b'0')),
            _ => (),
        };
    }
    unreachable!("parse_line end of function");
}

fn compare(lhs: &Elem, rhs: &Elem) -> Ordering {
    let mut lhs_v = &vec![];
    let mut rhs_v = &vec![];

    if let List(a) = lhs {
        lhs_v = a;
    }
    if let List(a) = rhs {
        rhs_v = a;
    }

    for i in 0..lhs_v.len() {
        if i >= rhs_v.len() {
            return Ordering::Greater;
        }
        let compare_result = match (&lhs_v[i], &rhs_v[i]) {
            (List(_), List(_)) => compare(&lhs_v[i], &rhs_v[i]),
            (List(_), Number(b)) => compare(&lhs_v[i], &List(vec![Number(*b)])),
            (Number(a), List(_)) => compare(&List(vec![Number(*a)]), &rhs_v[i]),
            (Number(a), Number(b)) => a.cmp(b),
        };
        if compare_result != Ordering::Equal {
            return compare_result;
        }
    }

    if rhs_v.len() > lhs_v.len() {
        return Ordering::Less;
    }
    Ordering::Equal
}

fn main() {
    let file_path = String::from("../aoc_13a/input");
    println!("In file {}", file_path);
    let mut data: Vec<Elem> = vec![];

    if let Ok(lines) = read_lines(file_path) {
        // Consumes the iterator, returns an (Optional) String
        for (i, line) in lines.enumerate() {
            if let Ok(ip) = line {
                match i % 3 {
                    0..=1 => data.push(parse_line(&ip, 1).0),
                    _ => (),
                }
            }
        }
    }

    data.push(List(vec![List(vec![Number(2)])]));
    data.push(List(vec![List(vec![Number(6)])]));
    data.sort_by(|a, b| compare(a, b));

    let mut a = None;
    let mut b = None;
    for i in 0..data.len() {
        if compare(&data[i], &List(vec![List(vec![Number(2)])])) == Ordering::Equal {
            a = Some(i + 1);
        }
        if compare(&data[i], &List(vec![List(vec![Number(6)])])) == Ordering::Equal {
            b = Some(i + 1);
        }
    }
    let a = a.unwrap();
    let b = b.unwrap();
    println!("{} {} {}", a, b, a * b);
}

fn read_lines<P>(filename: P) -> io::Result<io::Lines<io::BufReader<File>>>
where P: AsRef<Path>, {
    let file = File::open(filename)?;
    Ok(io::BufReader::new(file).lines())
}
